<?php

namespace Guzzle\Tests\Batch;

use Guzzle\Batch\Batch;
use Guzzle\Batch\FlushingBatch;

/**
 * @covers Guzzle\Batch\FlushingBatch
 */
class FlushingBatchTest extends \Guzzle\Tests\GuzzleTestCase {
	public function testFlushesWhenSizeMeetsThreshold() {
		$t = $this->getMock('Guzzle\Batch\BatchTransferInterface', ['transfer']);
		$d = $this->getMock('Guzzle\Batch\BatchDivisorInterface', ['createBatches']);

		$batch = new Batch($t, $d);
		$queue = $this->readAttribute($batch, 'queue');

		$d->expects($this->exactly(2))
			->method('createBatches')
			->will($this->returnCallback(function () use ($queue) {
				$items = [];
				foreach ($queue as $item) {
					$items[] = $item;
				}
				return [$items];
			}));

		$t->expects($this->exactly(2))
			->method('transfer');

		$flush = new FlushingBatch($batch, 3);
		$this->assertEquals(3, $flush->getThreshold());
		$flush->setThreshold(2);
		$flush->add('foo')->add('baz')->add('bar')->add('bee')->add('boo');
		$this->assertEquals(1, count($flush));
	}
}
